@@ -1,5 +1,6 @@
 from qbittorrentapi.decorators import Alias
 from qbittorrentapi.decorators import aliased
+from qbittorrentapi.decorators import handle_hashes
 from qbittorrentapi.decorators import login_required
 from qbittorrentapi.decorators import response_json
 from qbittorrentapi.definitions import APINames
@@ -9,11 +10,11 @@ from qbittorrentapi.request import Request
 
 
 class SyncMainDataDictionary(Dictionary):
-    pass
+    """Response for :meth:`~SyncAPIMixIn.sync_maindata`"""
 
 
 class SyncTorrentPeersDictionary(Dictionary):
-    pass
+    """Response for :meth:`~SyncAPIMixIn.sync_torrent_peers`"""
 
 
 class Sync(ClientCache):
@@ -30,7 +31,7 @@ class Sync(ClientCache):
         >>> # this will automatically request the changes since the last call
         >>> md = client.sync.maindata.delta()
         >>> #
-        >>> torrentPeers= client.sync.torrentPeers(hash="...'", rid='...')
+        >>> torrentPeers = client.sync.torrentPeers(hash="...'", rid='...')
         >>> torrent_peers = client.sync.torrent_peers(hash="...'", rid='...')
     """
 
@@ -79,7 +80,15 @@ class Sync(ClientCache):
 
 @aliased
 class SyncAPIMixIn(Request):
-    """Implementation of all Sync API Methods."""
+    """
+    Implementation of all Sync API Methods.
+
+    :Usage:
+        >>> from qbittorrentapi import Client
+        >>> client = Client(host='localhost:8080', username='admin', password='adminadmin')
+        >>> maindata = client.sync_maindata(rid="...")
+        >>> torrent_peers = client.sync_torrent_peers(torrent_hash="...'", rid='...')
+    """
 
     @property
     def sync(self):
@@ -100,13 +109,13 @@ class SyncAPIMixIn(Request):
         Retrieves sync data.
 
         :param rid: response ID
-        :return: dictionary response
-            Properties: https://github.com/qbittorrent/qBittorrent/wiki/WebUI-API-(qBittorrent-4.1)#get-main-data
+        :return: :class:`SyncMainDataDictionary` - https://github.com/qbittorrent/qBittorrent/wiki/WebUI-API-(qBittorrent-4.1)#get-main-data
         """
         data = {"rid": rid}
         return self._post(_name=APINames.Sync, _method="maindata", data=data, **kwargs)
 
     @Alias("sync_torrentPeers")
+    @handle_hashes
     @response_json(SyncTorrentPeersDictionary)
     @login_required
     def sync_torrent_peers(self, torrent_hash=None, rid=0, **kwargs):
@@ -117,10 +126,9 @@ class SyncAPIMixIn(Request):
 
         :param torrent_hash: hash for torrent
         :param rid: response ID
-        :return: Dictionary of torrent sync data.
-            Properties: https://github.com/qbittorrent/qBittorrent/wiki/WebUI-API-(qBittorrent-4.1)#get-torrent-peers-data
+        :return: :class:`SyncTorrentPeersDictionary` - https://github.com/qbittorrent/qBittorrent/wiki/WebUI-API-(qBittorrent-4.1)#get-torrent-peers-data
         """
-        data = {"hash": torrent_hash or kwargs.pop("hash"), "rid": rid}
+        data = {"hash": torrent_hash, "rid": rid}
         return self._post(
             _name=APINames.Sync, _method="torrentPeers", data=data, **kwargs
         )
